import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { ISavedSearches, SavedSearches } from '../saved-searches.model';

import { SavedSearchesService } from './saved-searches.service';

describe('SavedSearches Service', () => {
  let service: SavedSearchesService;
  let httpMock: HttpTestingController;
  let elemDefault: ISavedSearches;
  let expectedResult: ISavedSearches | ISavedSearches[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(SavedSearchesService);
    httpMock = TestBed.inject(HttpTestingController);

    elemDefault = {
      id: 0,
      name: 'AAAAAAA',
      jsonQuery: 'AAAAAAA',
    };
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = Object.assign({}, elemDefault);

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(elemDefault);
    });

    it('should create a SavedSearches', () => {
      const returnedFromService = Object.assign(
        {
          id: 0,
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.create(new SavedSearches()).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a SavedSearches', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          name: 'BBBBBB',
          jsonQuery: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.update(expected).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a SavedSearches', () => {
      const patchObject = Object.assign({}, new SavedSearches());

      const returnedFromService = Object.assign(patchObject, elemDefault);

      const expected = Object.assign({}, returnedFromService);

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of SavedSearches', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          name: 'BBBBBB',
          jsonQuery: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toContainEqual(expected);
    });

    it('should delete a SavedSearches', () => {
      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult);
    });

    describe('addSavedSearchesToCollectionIfMissing', () => {
      it('should add a SavedSearches to an empty array', () => {
        const savedSearches: ISavedSearches = { id: 123 };
        expectedResult = service.addSavedSearchesToCollectionIfMissing([], savedSearches);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(savedSearches);
      });

      it('should not add a SavedSearches to an array that contains it', () => {
        const savedSearches: ISavedSearches = { id: 123 };
        const savedSearchesCollection: ISavedSearches[] = [
          {
            ...savedSearches,
          },
          { id: 456 },
        ];
        expectedResult = service.addSavedSearchesToCollectionIfMissing(savedSearchesCollection, savedSearches);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a SavedSearches to an array that doesn't contain it", () => {
        const savedSearches: ISavedSearches = { id: 123 };
        const savedSearchesCollection: ISavedSearches[] = [{ id: 456 }];
        expectedResult = service.addSavedSearchesToCollectionIfMissing(savedSearchesCollection, savedSearches);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(savedSearches);
      });

      it('should add only unique SavedSearches to an array', () => {
        const savedSearchesArray: ISavedSearches[] = [{ id: 123 }, { id: 456 }, { id: 78948 }];
        const savedSearchesCollection: ISavedSearches[] = [{ id: 123 }];
        expectedResult = service.addSavedSearchesToCollectionIfMissing(savedSearchesCollection, ...savedSearchesArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const savedSearches: ISavedSearches = { id: 123 };
        const savedSearches2: ISavedSearches = { id: 456 };
        expectedResult = service.addSavedSearchesToCollectionIfMissing([], savedSearches, savedSearches2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(savedSearches);
        expect(expectedResult).toContain(savedSearches2);
      });

      it('should accept null and undefined values', () => {
        const savedSearches: ISavedSearches = { id: 123 };
        expectedResult = service.addSavedSearchesToCollectionIfMissing([], null, savedSearches, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(savedSearches);
      });

      it('should return initial array if no SavedSearches is added', () => {
        const savedSearchesCollection: ISavedSearches[] = [{ id: 123 }];
        expectedResult = service.addSavedSearchesToCollectionIfMissing(savedSearchesCollection, undefined, null);
        expect(expectedResult).toEqual(savedSearchesCollection);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
